// HPCSE I: pi computation, MPI version: reduction implemented with MPI_Send()/MPI_Recv()
#include <mpi.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char** argv)
{
	MPI_Init(&argc, &argv);

	int rank, size;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	unsigned long nterms;

	nterms = 100000000;

	double const step = (nterms+0.5l) /  size;
 
	double sum=0.;

	// do just one piece on each rank
	double localsum=0.;
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms;

	for (unsigned long t = start; t < end; ++t)
		localsum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	if (rank == 0) 
	{
		sum = localsum;
		double tmp;
		for (int i = 1; i < size; i++) {
			MPI_Recv(&tmp, 1, MPI_DOUBLE, i, 7777, MPI_COMM_WORLD, MPI_STATUS_IGNORE); // replace i with MPI_ANY_SOURCE
			sum += tmp;
		}
	}
	else
		MPI_Send(&localsum, 1, MPI_DOUBLE, 0, 7777, MPI_COMM_WORLD);	

	if (rank == 0)
		printf("rank %d: pi= %lf\n", rank, 4.*sum);

	MPI_Finalize();

	return 0;
}
