// HPCSE I: pi computation, MPI version: using MPI_Reduce()
#include <mpi.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char** argv)
{
	MPI_Init(&argc, &argv);

	int rank, size;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	unsigned long nterms;

	nterms = 100000000;

	double const step = (nterms+0.5l) /  size;
 

	// do just one piece on each rank
	double sum=0.;
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms;

	for (unsigned long t = start; t < end; ++t)
		sum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	MPI_Reduce(rank==0? MPI_IN_PLACE:&sum, &sum, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

	if (rank == 0)
		printf("rank %d: pi= %lf\n", rank, 4.*sum);

	MPI_Finalize();

	return 0;
}
