// HPCSE I: pi computation, MPI version: MPI_Reduce() implemented with MPI_Gather()
#include <mpi.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char** argv)
{
	MPI_Init(&argc, &argv);

	int rank, size;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	unsigned long nterms;

	nterms = 100000000;

	double const step = (nterms+0.5l) /  size;
 
	double sum=0.;

	// do just one piece on each rank
	double localsum=0.;
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms;

	for (unsigned long t = start; t < end; ++t)
		localsum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	double tmp[size];
	MPI_Gather(&localsum, 1, MPI_DOUBLE, tmp, 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);  // "tmp, 1": number of elements for any *single* receive (significant only at root) 
	if (rank == 0) 
	{
		sum = tmp[0];
		for (int i = 1; i < size; i++)
			sum += tmp[i];
	}

	if (rank == 0)
		printf("rank %d: pi= %lf\n", rank, 4.*sum);

	MPI_Finalize();

	return 0;
}
