// HPCSE I: pi computation, MPI version: broadcasting with MPI_Send/Recv()
#include <mpi.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char** argv)
{
	MPI_Init(&argc, &argv);

	int rank, size;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	unsigned long nterms;

	if (rank == 0) nterms = 100000000;	// for some application-specific reason, this is set only on rank 0

	// we need to share the value of nterms with the other ranks
	if (rank == 0) 
	{
		for (int i = 1; i < size; i++)
			MPI_Send(&nterms, 1, MPI_UNSIGNED_LONG, i, 8888, MPI_COMM_WORLD);	
	}
	else
		MPI_Recv(&nterms, 1, MPI_UNSIGNED_LONG, 0, 8888, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

	double const step = (nterms+0.5l) /  size;
 
	double sum=0.;

	// do just one piece on each rank
	double localsum=0.;
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms;

	for (unsigned long t = start; t < end; ++t)
		localsum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	MPI_Reduce(&localsum, &sum,1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

	if (rank == 0)
		printf("rank %d: pi= %lf\n", rank, 4.*sum);

	MPI_Finalize();

	return 0;
}
