// HPCSE I: sendto()/recvfrom() routines for exchanging messages between threads that do not share memory directly
// Note: a more modular version of the previous implementation (omp_sendrecv1.c)
#include <omp.h>
#include <stdio.h>
#include <unistd.h>

#define ACT_SEND 0
#define ACT_RECV 1
void sendrecv(double *data, int rank, int action)
{
	static double buffer_data;
	static int buffer_ready;

	if (action == ACT_SEND) 
	{

		#pragma omp critical
		{
		buffer_data = *data;
		buffer_ready = 1;
		}
	}
	else	// ACT_RECV
	{
		while (buffer_ready != 1) ;
		*data = buffer_data;
	}
}
 
void sendto(double *x, int rank)
{
	sendrecv(x, rank, ACT_SEND);
}

void recvfrom(double *y, int rank)
{
	sendrecv(y, rank, ACT_RECV);
}

int main(int argc, char** argv)
{
#pragma omp parallel
{
	int rank = omp_get_thread_num();

	if (rank == 0)
	{
		double x=99; 
		sendto(&x, 1);
	}
	else if (rank == 1)
	{
		double y=0;
		recvfrom(&y, 0);
		printf("rank %d: y = %lf\n", rank, y);
	}
}

  return 0;
}
