// HPCSE I: pi computation, parallel code, version 2: parallel region without worksharing construct and manual reduction with atomic
#include <omp.h>
#include <stdio.h>
#include <unistd.h>

int main(int argc, char** argv)
{

#pragma omp parallel
{
    double sum=0.;
    
	int rank = omp_get_thread_num();
	int size = omp_get_num_threads();
	double localsum=0.;
  
	unsigned long const nterms = 100000000;
	double const step = (nterms+0.5l) /  size;
  
	// do just one piece on each rank
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms; 

	for (unsigned long t = start; t < end; ++t)
		localsum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	#pragma omp atomic
	sum += localsum;

	#pragma omp barrier
	if (rank==0) // only one prints
		printf("rank %d: pi= %lf\n", rank, 4.*sum);

}
  
	return 0;
}
