// HPCSE I: pi computation, parallel code, version 3b: a single parallel region from the very beginning, without worksharing construct, reductions, shared variables
// Note: solution code 
#include <omp.h>
#include <stdio.h>
#include <unistd.h>

void broadcast_ulong(unsigned long *value)
{
	static unsigned long broadcast_buffer;
	int rank = omp_get_thread_num();

	if (rank == 0)
	{
		broadcast_buffer = *value; 
	}
	#pragma omp barrier
	if (rank != 0)
	{
		*value = broadcast_buffer;
	}
}

void reduce_double(double *local_sum, double *sum)
{
	static double reduce_buffer[8192]; // big enough 
	int rank = omp_get_thread_num();
	int size = omp_get_num_threads();

	reduce_buffer[rank] = *local_sum;

	#pragma omp barrier

	*sum = 0;
	if (rank == 0)
	{
		for (int i = 0; i < size; i++)
			*sum += reduce_buffer[i];
	}
}

int main(int argc, char** argv)
{
#pragma omp parallel
{
	int rank = omp_get_thread_num();
	int size = omp_get_num_threads();

	unsigned long nterms;

	if (rank == 0) nterms = 100000000;

	broadcast_ulong(&nterms);

	double const step = (nterms+0.5l) /  size;
 
	double sum=0.;

	// do just one piece on each rank
	double localsum=0.;
	unsigned long start = rank * step;
	unsigned long end = (rank+1) * step;
	if (rank == size -1) end = nterms;

	for (unsigned long t = start; t < end; ++t)
		localsum += (1.0 - 2* (t % 2)) / (2*t + 1);
  
	#pragma omp barrier

	reduce_double(&localsum, &sum);

	if (rank == 0)
		printf("rank %d: pi= %lf\n", rank, 4.*sum);
}

	return 0;
}
